<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetHelperFolder extends JObject
{
	
	protected $_db;
	public $id;
	public $name;
	public $parent;
	public $type;
	public $ftp;
	public $alert;
	public $uploadBase;
	public $uploadPath;
	protected static $instance;
	
	
	public static function getInstance( $id ) {
		if( !isset(self::$instance[$id] ) ) {
			self::$instance[$id] = new self( $id );
		}
		return self::$instance[$id];
	}
	

	public function __construct( $id ) {
		
		$this->_db 			= JFactory::getDbo();
		$this->id			= $id;
		$this->name 		= null;
		$this->parent 		= 0;
		$this->type			= 'pv';
		$this->ftp			= null;
		$this->alert 		= 0;
		$this->uploadBase 	= JExtranetHelper::getSystem()->getUploadFolder();
		$this->getFolder();
	}
	

	/*
	 * Method to get folder details
	 */
	protected function getFolder() {
		
		$query = $this->_db->getQuery( true );
		$query->select('a.fd_id AS id, a.fd_parent AS parent, a.fd_name AS name,a.fd_type AS type, a.fd_name_ftp AS ftp, a.fd_alerts AS alert, a.published');
		$query->from('#__extranet_folders AS a');
		$query->where('a.fd_id = ' . $this->id);

		$this->_db->setQuery($query);
		$this->setProperties( $this->_db->loadAssoc() );	
		$this->uploadPath	= $this->uploadBase . $this->ftp;	
	}


	/*
	 * Method to get all subfolders contained by a main folder
	 * @return		array 		an array of subfolders
	 */
	public function getSubfolders ( $recursive = false ) {

		$query	= $this->_db->getQuery(true);
	
		// get all folders
		$query->select('a.fd_id AS id, a.fd_name AS name, a.fd_parent AS parent, a.fd_name_ftp as ftp');
		$query->from('#__extranet_folders AS a ');
		$query->order('a.fd_parent ASC');
		$this->_db->setQuery( $query );
		$rez = $this->_db->loadObjectList ();
			
		$sub		= array();
		$subfolders = $this->get_child_nodes( $this->id, 0, $rez, $sub, $recursive );
		
		return $subfolders;
	}
	
	
	protected function get_child_nodes($parent, $level, $folders, &$subfolders, $recursive ) {
		
		foreach ( $folders as $folder ) {
			if ( $folder->parent == $parent ) {
				$folder->deep = $level;
				$subfolders[] = $folder;
				if ( $recursive) {
					$this->get_child_nodes( $folder->id, $level+1, $folders, $subfolders, $recursive );
				}
			}
		}
		
		return $subfolders;
	}
	

	// method to see if this folder is parent of needle
	public function isParentFor( $needle ){
			
		$childrens = $this->getSubfolders( true );
		foreach ( $childrens as $children ) {
			if ( $children->id == $needle ) {
				return true;
			}
		}
		return false;
	}
	
	
	/*
	 * Method to return parent ID, if recursive will return all parents until the root folder
	 * @return		array 		array containing id of all the parents of this folder
	 */
	public function getParent ($recursive = false) 
	{
		
		// Initialize variables.
	 	$parents = array();
				
		$parents[] 	= $this->parent;
	
		if ($recursive) 
		{
			$recursive = $this->parent;
			while ($recursive) 
			{ 
				$folder 	= JExtranetHelper::getFolder($recursive);
				$parents[] 	= (int) $folder->parent;
				$recursive 	= (int) $folder->parent;
			}
		}
		return $parents;
	}


	public function getMainParent()
	{
		if ($this->parent == 0)
		{
			return $this->id;
		}
		else 
		{
			return self::getInstance($this->parent)->getMainParent();
		}
	}
	
	
	
	// method to return the path to current folder
	public function getRoute() {
		
		$nodes		= array();
		
		if ( !$this->id ) {
			return $nodes;
		}
		
		$folderpath = $this->getParent( true );
		
		foreach ( $folderpath as $folder ) {
			$node = new stdclass;
			$node->id 		= $folder;
			$node->name 	= JExtranetHelper::getFolder( $folder )->name;
			$node->parent	= JExtranetHelper::getFolder( $folder )->parent;
			$node->same		= $this->getSameLevel( $node->parent, $node->id );
			$nodes[] = $node;
		}
				
		$nodes = array_reverse ( $nodes );
		
		return $nodes;	
	}
	
	
	// method to return the folders located inside the same folder parent
	public function getSameLevel( $parent, $exclude = 0 ) {
		
		$query	= $this->_db->getQuery(true);
				
		$query->select('a.fd_id AS id, a.fd_name AS name');
		$query->from('#__extranet_folders AS a ');
		$query->where( 'a.fd_parent = ' . $parent );
		$query->where( 'a.fd_id != ' . (int) $exclude );
		$query->order('a.fd_id ASC');
		
		$this->_db->setQuery( $query );
		$rez = $this->_db->loadObjectList ();
		return $rez;
	}
	
	
	// delete the folder and included files from database / server 
	public function delete() {
		
		// first try to delete from server
		$path 	= JExtranetHelper::getSystem()->getUploadFolder() . DIRECTORY_SEPARATOR . $this->ftp;
		if (JFolder::exists($path)) {
			if (!JFolder::delete($path)) {
				return false;
			}
		}
		
		// delete the folder and files from database
		$query = $this->_db->getQuery( true );
		$query->delete();
		$query->from( $this->_db->quoteName('#__extranet_folders') );
		$query->where( $this->_db->quoteName('fd_id') . '=' . $this->id );
		$this->_db->setQuery($query);
		$this->_db->query();
		
		$query->clear();
		$query->delete();
		$query->from( $this->_db->quoteName('#__extranet_files') );
		$query->where( $this->_db->quoteName('f_parent') . ' = ' . $this->id );
		$this->_db->setQuery($query);
		$this->_db->query();
		
		return true;
	}
	
	
}
