<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.utilities.date');

class JExtranetHelperSystem extends JExtranetHelper
{
	
	public $params;
	protected static $uploadPath;
	
	public function __construct () {
		$this->params = JComponentHelper::getParams('com_jextranet');
		parent::__construct();
	}


	public function getDateToMySQL() 
	{

		$jdate 	= new JDate();
		$date 	= $jdate->toSql(true);

		return $date;
	}
	

	public static function getActions() {
		$user	= JFactory::getUser();
		$result	= new JObject;

		$assetName = 'com_jextranet';

		$actions = array(
			'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.state', 'core.delete'
		);

		foreach ($actions as $action) {
			$result->set($action,	$user->authorise($action, $assetName));
		}

		return $result;
	}



	public static function addSubmenu ( $view ) {

		JHtmlSidebar::addEntry(
                       '<i class="icon icon-folder"></i> ' . JText::_( 'COM_JEXTRANET_FOLDERS' ),
                        'index.php?option=com_jextranet&view=folders', $view == 'folders' || $view == 'foldersf' );
                   
		JHtmlSidebar::addEntry(
						'<i class="icon icon-user"></i> ' . JText::_( 'COM_JEXTRANET_USERS' ),
                        'index.php?option=com_jextranet&view=users', $view == 'users'  || $view =='userlogs' || $view =='userfiles');           
                        
		JHtmlSidebar::addEntry(
                        '<i class="icon icon-users"></i> ' .JText::_( 'COM_JEXTRANET_GROUPS' ),
                        'index.php?option=com_jextranet&view=groups', $view == 'groups');     
                        
		JHtmlSidebar::addEntry(
                       '<i class="icon icon-flag"></i> ' . JText::_( 'COM_JEXTRANET_ALERTS' ),
                        'index.php?option=com_jextranet&view=alerts', $view == 'alerts');                    

		JHtmlSidebar::addEntry(
                        '<i class="icon icon-envelope"></i> ' .JText::_( 'COM_JEXTRANET_MESSAGES' ),
                        'index.php?option=com_jextranet&view=messages', $view == 'messages');     
                        
		JHtmlSidebar::addEntry(
                        '<i class="icon icon-warning"></i> ' . JText::_( 'COM_JEXTRANET_STATISTICS' ),
                        'index.php?option=com_jextranet&view=statistics', $view == 'statistics');   
                        
		JHtmlSidebar::addEntry(
                        '<i class="icon icon-info"></i> ' . JText::_( 'COM_JEXTRANET_ABOUT' ),
                        'index.php?option=com_jextranet&view=about', $view == 'about');      
	}
	
	
	public static function parseSize( $size ) {
		if ($size < 1024) {
			return $size . ' ' . JText::_('COM_JEXTRANET_FILESIZE_BYTES');
		}
		elseif ($size < 1024 * 1024) {
			return sprintf('%01.2f', $size / 1024.0) . ' ' .   JText::_('COM_JEXTRANET_FILESIZE_KILOBYTES') ;
		}
		else {
			return sprintf('%01.2f', $size / (1024.0 * 1024)) . ' ' . JText::_('COM_JEXTRANET_FILESIZE_MEGABYTES') ;
		}
	}



	public static function countFiles($dir) {
		
		$total_file = 0;
		$total_dir = 0;

		if (is_dir($dir)) {
			$d = dir($dir);

			while (false !== ($entry = $d->read())) {
				if (substr($entry, 0, 1) != '.' && is_file($dir . DIRECTORY_SEPARATOR . $entry) && strpos($entry, '.html') === false && strpos($entry, '.php') === false) {
					$total_file++;
				}
				if (substr($entry, 0, 1) != '.' && is_dir($dir . DIRECTORY_SEPARATOR . $entry)) {
					$total_dir++;
				}
			}

			$d->close();
		}

		return array ($total_file, $total_dir);
	}
	

	/*
	 * name: 	getExtensionDetails
	 * @param 	string $extension_name 	the name of the searched extension
	 * @return 	array with extension details ( version, copyright, website, created )
	 */
	public static function getExtensionDetails () {
		$xml = simplexml_load_file(JPATH_COMPONENT . '/jextranet.xml');
		return $xml;
	}


	/*
	 * Method to get a list with all the groups
	 */
	public static function getGroups () {

		$db		= JFactory::getDbo();
		$groups	= array();
		
		$query 		= $db->getQuery(true);
		$query->select( 'a.g_name AS text, a.g_id AS value' );
		$query->from( '#__extranet_users_groups AS a' );
		$db->setQuery($query);
		$groups 	= $db->loadObjectList();

		return $groups;
	}
	

	/*
	 * Method to get a list with all the users
	 */
	public static function getUsers () {

		$db		= JFactory::getDbo();
		$users		= array();
		
		$query 		= $db->getQuery(true);
		$query->select( 'a.u_id AS value, b.name AS text, b.email' );
		$query->from( '#__extranet_users AS a' );
		$query->join('inner','#__users AS b ON a.u_id = b.id ');
		$query->where('a.block = 1');

		$db->setQuery($query);
		$users 	= $db->loadObjectList();

		return $users;
	}


	public static function getGroupUsers($group) {

		$db		= JFactory::getDbo();
		$query 		= $db->getQuery(true);

		$query->select('a.u_id');
		$query->from('#__extranet_users AS a');
		$query->join('inner','#__users AS b ON a.u_id = b.id ');
		$query->where('a.g_id = ' . (int) $group );

		$db->setQuery ( $query );
		return $db->loadColumn();
	}

	/*
	 * Method to get a list with all the users who aren't extranet users (to be displayed into the popup when adding a new user)
	 * 
	 * name: 			getUsers
	 * @param			
	 * @return			object 
	 */
	public static function getNonUsers() {
		
		$db	= JFactory::getDbo();
		$users 	= array();
		$query 	= $db->getQuery(true);
		
		$query->select( ' a.id AS value, a.name AS text ' );
		$query->from( '#__users AS a' );
		$query->where( ' a.id NOT IN ( SELECT b.u_id FROM #__extranet_users AS b ) ' );
		$query->order( 'a.id DESC' );
		$db->setQuery($query);
		$users = $db->loadObjectList ();
		
		return $users;
	}



	/*
	 * Method to get the default path for storing J!Extranet folderss
	 */
	public function getUploadFolder () {

		if (!isset(self::$uploadPath)) {
			self::$uploadPath = $this->params->get(	'defaultUploadLocation',   
								JPATH_COMPONENT . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'extranet' );
			self::$uploadPath = ( self::$uploadPath == '/' ) ? JPATH_COMPONENT . DIRECTORY_SEPARATOR . 'media' . DIRECTORY_SEPARATOR . 'extranet' : self::$uploadPath;

		} 

		return self::$uploadPath;
	}

}
?>
