<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');

class JExtranetModelFile extends JModelAdmin
{

	protected $text_prefix = 'COM_JEXTRANET';
	protected $baseUpload;

	public function __construct( $config = array() ) {
		
		$this->baseUpload = JExtranetHelper::getSystem()->getUploadFolder();
		parent::__construct($config);
	}

	public function getTable($type = 'File', $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_jextranet.file', 'file', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		
		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_jextranet.edit.file.data', array());

		if (empty($data)) {
			$data = $this->getItem();
		}

		return $data;
	}



	protected function prepareTable( $table ) {
		
		if (empty($table->f_id)) {
				
			// set ordering
			if (empty($table->ordering)) {
				
				$data 	= JRequest::getVar('jform',array(),'post','array');
				$parent	= $data['f_parent'];
				$query 	= 'SELECT MAX(a.ordering) FROM #__extranet_files AS a WHERE a.f_parent = ' . $parent;
				$this->_db->setQuery ($query);
				$max = $this->_db->loadResult();
				$table->ordering = $max+1;
			}
			
			// set the id of the owner
			$table->f_created_by = JFactory::getUser()->get('id');
			
			// set creation date
			$table->f_created = JExtranetHelper::getSystem()->getDateToMySQL();
		}
	}
	
	
	/*
	 * Method to save the file and to send alert if available
	 * @param		array $data 	file details from the from 
	 * @return		boolean 	true if success, false on failure
	 */
	public function save( $data ) {
	
		if( $data['f_alias'] ) { 
			if ($this->isLink( $data['f_alias'] )) {
				$file = $data['f_alias'];
			} else {
				$file = $this->baseUpload . DIRECTORY_SEPARATOR . $data['f_alias'];
			}
		
			$extensie = JFile::getExt( $file ); 
			$not_alphanumeric = '/[^A-Za-z0-9]/';
			if (preg_match( $not_alphanumeric, $extensie )) {
				$extensie = '';
			}

			clearstatcache();
			$data['f_type'] 	= $extensie;
			$data['f_size'] 	= @filesize($file);
		}	

		if (parent::save($data)) {

			// if saved data try to send alert
			if ( !$data['f_id'] &&  $data['published'] == 1 ) {
					
				$params	= JComponentHelper::getParams('com_jextranet');
	
				$notif 			= new stdclass;
				$notif->users 	= $this->getAuthorizedUsers( $data['f_parent'] );
				$notif->files	= $data['f_title'];
				$notif->folder	= $data['f_parent'];
				$notif->html 	= $params->get('alertFormat', 0) ? false : true;
				
				// get all alerts that are set for a certain folder
				$alerts			= $this->getAlert( $folder );
				foreach ( $alerts as $alert ) {
					
					$notif->subject	= $alert->subject;
					$notif->message	= $alert->message;
					$this->sendAlert( $notif );
				}
			}

			return true;
		}
		return false;
	}
	
	
	// retrieve an array with alerts to be used for a certain folder
	public function getAlert( $folder ) {
		
		$now	= JExtranetHelper::getSystem()->getDateToMySQL();
		$query 	= $this->_db->getQuery( true );
		$regex 	= $this->_db->quote( '"' . (int) $folder . '"' );
		
		$query->select('a.a_description AS message, a.a_subject AS subject');
		$query->from('#__extranet_alerts AS a');
		$query->where( 'a_created <= '. $this->_db->Quote( $now ) );
		$query->where('((a.a_publish_up = "0000-00-00 00:00:00") OR (a.a_publish_up <= '. $this->_db->Quote( $now ) .'))');
		$query->where('((a.a_publish_down = "0000-00-00 00:00:00") OR (a.a_publish_down > '. $this->_db->Quote( $now ) .'))');
		$query->where('a.params REGEXP ' . $regex );

		$this->_db->setQuery( $query );
		return $this->_db->loadObjectList();
		
	}
	
	// retrieve all authorized users that can receive an alert attached to a folder
	public function getAuthorizedUsers( $folder ) {
		
		$alerted	= array();
		$list 		= JExtranetHelper::getSystem()->getUsers ();
		foreach ( $list as $user ) {
			$authLevel	= JExtranetHelper::getUser( $user->value )->getAuthorisationLevel( $folder ); 
			if ( in_array( 2, $authLevel ) || in_array( 3, $authLevel ) ) { 
				$alerted[] = JFactory::getUser( $user->value );
			}
		}
		return $alerted;
	}
	

	protected function sendAlert( $data ) {
	
		$params	= JComponentHelper::getParams('com_jextranet');
		
		$alerter = JExtranetHelper::getAlerter( $data ); 
		$alerter->prepare();
		
		if ( $params->get('sendAlertsMethods', 0 ) == 0 || $params->get('sendAlertsMethods', 0 ) == 2 ) {
			$alerter->sendEmail();
		}
		
		if ( $params->get('sendAlertsMethods', 0 ) == 1 || $params->get('sendAlertsMethods', 0 ) == 2 ) {
			$alerter->sendMessage();
		}
	}
	
	
	
	
	/*
	 * Method to delete selected files 
	 * 
	 * name: 		deleteFiles
	 * @param		array $ids		id of the files to be deleted	
	 * @return		boolean
	 */
	public function deleteFiles ( &$ids ) {
		
		$table	= $this->getTable();
		
		foreach ( $ids as $id ) {
			
			if ($table->load($id)) {
				
				if (!$table->delete( $id )) {
					$this->setError($table->getError());
					return false;
					
				} else {

					if ( !$this->isLink($table->f_alias) ) {
						$path 		= 	$this->baseUpload . DIRECTORY_SEPARATOR . $table->f_alias;
						$preview 	= 	$this->baseUpload . DIRECTORY_SEPARATOR . $table->f_preview;
				
						if (!$this->getFileDependencies($table->f_alias)) {
							if (JFile::exists($path)) {
								if (!JFile::delete($path)) {
									$this->setError ( JText::_('COM_JEXTRANET_FOLDERS_ERROR_NO_FOLDER_FTP_DELETED') . $table->f_alias );
									return false;
								} 
							}
						
							if (JFile::exists($preview)) {
								if (!JFile::delete($preview)) {
									$this->setError ( JText::_('COM_JEXTRANET_FOLDERS_ERROR_NO_FOLDER_FTP_DELETED') . $table->f_preview );
									return false;
								} 
							}
						}
					}
				}	
			}
		}

		return true;
	}
	
	
	
	/*
	 * Method to check if more files are using the same physical file
	 * 
	 * name: 		getFileDependencies
	 * @param		string $file		the file alias (path on ftp)
	 * @return		boolean				true if any other file is using the same physical file
	 */
	protected function getFileDependencies ( $file ) {
		
		$query = 	' SELECT COUNT(a.f_id) AS count ' .
					' FROM #__extranet_files AS a ' .
					' WHERE a.f_alias = ' . $this->_db->Quote($file);
				
		$this->_db->setQuery ( $query );
		$dependencies = $this->_db->loadResult();

		if (empty($dependencies)) {
			return false;
		} else {
			return true;
		}
	}
	
	
	// we need this to remove user files
	public function getUTable($type = 'Userfiles', $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}
	
	
	
	/*
	 * Method to delete selected userfiles 
	 * 
	 * name: 		deleteUFiles
	 * @param		array $ids		id of the files to be deleted	
	 * @return		boolean
	 */
	public function deleteUFiles ( &$ids, $user ) {
		
		JClientHelper::setCredentialsFromRequest('ftp');
		
		$table	= $this->getUTable();
		
		foreach ( $ids as $id ) {
			
			if ($table->load($id)) {
				
				if (!$table->delete( $id )) {
					$this->setError($table->getError());
					return false;
					
				} else {
					$path 		= 	$this->baseUpload . DIRECTORY_SEPARATOR . 'extranet_ufiles' . DIRECTORY_SEPARATOR . $user . DIRECTORY_SEPARATOR . $table->f_alias;

					if (JFile::exists($path)) {
						if (!JFile::delete($path)) {
							$this->setError ( JText::_('COM_JEXTRANET_FOLDERS_ERROR_NO_FILES_FTP_DELETED') . ': ' . $table->f_alias );
							return false;
						} 
					}
				}	
			}
		}

		return true;
	}
	

	private function isLink ( $alias ) {

		if ( stripos($alias, 'http://' ) !== false || stripos($alias, 'https://' ) !== false ) {
			return true;
		}
		return false;
	}


	// used when list all files inside a folder (foldersf)
	public function orderdown ( $id ) {
		
		//get item order 
		$query = ' SELECT a.ordering, a.f_parent FROM #__extranet_files AS a WHERE a.f_id = ' . $id . ' LIMIT 1';
		$this->_db->setQuery($query);
		$item = $this->_db->loadObject();
			
		$query	= ' SELECT COUNT(a.f_id) FROM #__extranet_files AS a ' .	
				  ' WHERE a.f_parent = ' . $item->f_parent;
		$this->_db->setQuery($query);
		$max = $this->_db->loadResult();

		if ($max <= 1) {
			return;
		} else {
			// get current order list	
			$query = ' SELECT a.ordering FROM #__extranet_files AS a WHERE a.f_parent = ' . $item->f_parent . ' ORDER BY a.ordering ASC ';
			$this->_db->setQuery($query);
			$orders = $this->_db->loadColumn();
			
			// find the position of the item into the order list
			$position 	= array_search ( $item->ordering,$orders );

			if ($position < ($max-1)) {
				
				$query = ' UPDATE #__extranet_files AS a SET a.ordering = '. $orders[$position] . ' WHERE a.ordering = ' . ($orders[($position+1)]) . ' AND a.f_parent = ' . $item->f_parent;
				$this->_db->setQuery($query);
				$this->_db->Query();
		
				$query = ' UPDATE #__extranet_files AS a SET a.ordering = '. ($orders[($position+1)]) . ' WHERE a.f_id = ' . $id;
				$this->_db->setQuery($query);
				$this->_db->Query();

			}
		}
	}
	
	
	// used when list all files inside a folder (foldersf)
	public function orderup ( $id ) {
		
		$query = ' SELECT a.ordering, a.f_parent FROM #__extranet_files AS a WHERE a.f_id = ' . $id . ' LIMIT 1';
		$this->_db->setQuery($query);
		$item = $this->_db->loadObject();

		if ( $item->ordering == 1 ) {
			
			return;
			
		} else {
			// get current order list	
			$query = ' SELECT a.ordering FROM #__extranet_files AS a WHERE a.f_parent = ' . $item->f_parent . ' ORDER BY a.ordering ASC ';
			$this->_db->setQuery($query);
			$orders = $this->_db->loadColumn();
			
			// find the position of the item into the order list
			$position 	= array_search ( $item->ordering,$orders );
			
			// set the order of the upper item ($position-1) as $position
			$query = ' UPDATE #__extranet_files AS a SET a.ordering = '. $orders[$position] . ' WHERE a.ordering = ' . ($orders[($position-1)]) . ' AND a.f_parent = ' . $item->f_parent;
			$this->_db->setQuery($query);
			$this->_db->Query();
			
			// set the order for the selected item as 1 position up ($position-1)
			$query = ' UPDATE #__extranet_files AS a SET a.ordering = '. ($orders[($position-1)]) . ' WHERE a.f_id = ' . $id;
			$this->_db->setQuery($query);
			$this->_db->Query();
		}
	}
	
	public function getFile ( $fileid ) {
		
		clearstatcache();


		$file = new stdclass();
		
		if ( $fileid ) {

			$query = $this->_db->getQuery( true );
			$query->select('a.f_id AS id, a.f_title AS title, a.f_alias AS alias, a.f_size AS size, a.f_type AS type');
			$query->from('#__extranet_files AS a');
			$query->where('a.f_id = ' . $fileid);
			$this->_db->setQuery($query);
			$rez = $this->_db->loadObject();

			if (!empty($rez)) { 

				$file = $rez;
				$file->location = $this->baseUpload . $file->alias;

				if (JFile::exists( $file->location )) { 
					if (function_exists('mime_content_type')) {
						$file->mime = mime_content_type($file->location);
					} elseif (function_exists('finfo_open')) {
						$finfo 		= finfo_open(FILEINFO_MIME);
						$mimeType 	= @finfo_file($finfo, $file->location);
						finfo_close($finfo);
						if (strpos($mimeType, ';')) { 
							list($mimeType,) = explode(';', $mimeType);
							$file->mime = $mimeType;
						} 
					} elseif (!empty($mime_type[strtolower($file->type)])) {
						$file->mime = $mime_type[strtolower($file->type)];
					} else {
						$file->mime = 'application/octet-stream';
					}
				}
			}
		} 
		return $file; 
	}

	public function getHeaders( $file ) {

		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-Type: " . $file->mime);
		if ($file->size) {
			header("Content-Length: " .(string)($file->size) );
		}
		header('Content-Disposition: attachment; filename="' . $file->title .'.'.$file->type.'"');
		header("Content-Transfer-Encoding: binary\n");
	}


	// method to get a file content
	public function getContent( $file ) {
		ob_clean();
		flush();
		$data = JFile::read( $file->location );
		echo $data;
		ob_flush();
		flush(); 
		exit(0); 
	}
}
?>
