<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetModelGroup extends JModelAdmin
{

	protected $text_prefix = 'COM_JEXTRANET_GROUPS';
    protected $per_page = 50;

    public function getTable($type = 'Exgroup',
                            $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

    public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
        $form = $this->loadForm('com_jextranet.group', 'group',
                        array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)){
			return false;
		}
		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
        $data = JFactory::getApplication()
                    ->getUserState('com_jextranet.edit.group.data', array());

		if (empty($data)){
			$data = $this->getItem();
		}

		return $data;
	}

    /*
     * retrieve an object containing a list of items and nr of pages
     * @param $parentFolder     the id of folder ("all" for all folders)
     * @param $offset           view page number (from pagination)
     * @param $group            the group ID to calculate permissions
     * @param $refresh          
     *              * since vue will refresh all items during a response,
     *              if we are in the "All" area, permissions div would be
     *              hidden, since the data refresh doesn't maintain the state of
     *              permissions view
     *              * set to 1 to keep open permissions div of "all"
     */
    public function permissionsJsonData($parentFolder, $offset, $group, $refresh = 0)
    {
        $data = new stdclass;
        $items = array();
        $folders = $this->getFolders($parentFolder, $offset);
        $stored = $this->getStoredPermissions($group);
        
        if(!empty($folders)) {
            if ($parentFolder == 0) {
                $items = array_slice($folders,$offset*$this->per_page,
                                                        $this->per_page);

                // if we are on main folders page, page nr 1
                if ($offset == 0) {
                    $root = new stdclass;
                    $root->id = 'all';
                    $root->name = JText::_('COM_JEXTRANET_ALL');
                    $root->active = $refresh;
                    array_unshift($items, $root);
                }
            } else {
                $items = $folders;
            }
        }

        foreach ($items as &$item) {
            $item->active = (isset($item->active)) ? $item->active : 0;
            $item->permissions = $this->getPermissions($item->id, $stored);
        }
        $data->pages = ceil(count($folders)/$this->per_page);
        $data->items = json_encode($items);

        return $data;
    }

    protected function getFolders($parent = 0)
    {
        $query = $this->_db->getQuery(true);
        $query->select('fd_id AS id,fd_name AS name');
        $query->from('#__extranet_folders');
        $query->where('fd_parent = ' . $parent);
        $query->where('fd_type = '. $this->_db->quote('pv'));
        $query->where('published = 1');
        $query->order('ordering DESC');

        $this->_db->setQuery($query);
        return $this->_db->loadObjectList();
    }

	public function deleteGroups($ids){
		if (!empty($ids)){
			$table 	= $this->getTable();
			foreach ( $ids as $id ) {
				$id = (int) $id;
				if ($table->load($id)){
					$table->delete($id);
					$query = 'UPDATE #__extranet_users AS a SET a.g_id = 0 WHERE a.g_id= ' . $id;
					$this->_db->setQuery($query);
					$this->_db->query();
				}
			}
            return true;
		}
        return false;
	}

	public function getStoredPermissions($groupID) 
	{
		
		$query	= $this->_db->getQuery( true );
		$query->select ('a.g_params');
		$query->from('#__extranet_users_groups AS a');
		$query->where('a.g_id = ' . (int) $groupID );

		$this->_db->setQuery($query);
		$params	= $this->_db->loadResult();
		
		if(!empty($params))
		{
			$params = json_decode($params);
		}

		if (isset($params->permissions))
		{
			return $params->permissions;
		}
		return new stdclass;
	}

	public function getPermissions($folder, $stored_permissions)
    { 
		$params = JComponentHelper::getParams('com_jextranet');

		// if the STATUS is set to be public to all folders
		if ($params->get('folderDefaultStatus') == 2)
		{
			return array(2,3);
		}

		// if he has set permissions to all folders
		if (!empty($stored_permissions->all))
		{
			return $stored_permissions->all;
		}

		if ($folder == 'all')
		{
			 return array(0);
		}

		// if he has permissions custom
		if ($params->get('subfolderPermission') == 1)
		{	
			// if he has permissions 
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
			// if not default to the global permissions
			else
			{
				return $params->get('globalRights', array(1));
			}
		}

		// if not, he has inherited permissions
		$parent = JExtranetHelper::getFolder($folder)->getMainParent();

		if ($parent == $folder)
		{	
			if(isset($stored_permissions->$folder))
			{
				return $stored_permissions->$folder;
			}
		}
		else
		{
			if(isset($stored_permissions->$parent))
			{
				return $stored_permissions->$parent;
			}
		}

		return $params->get('globalRights', array(1));
	}
}
