<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');

class JExtranetModelMultiple extends JModelLegacy
{

	protected $text_prefix = 'COM_JEXTRANET';

	
	public function upload ( $folderId, $fileOrder) {
		
		// Set FTP credentials, if given
		JClientHelper::setCredentialsFromRequest('ftp');
		
		$post			= new stdClass();
				
		$post->error 	= null;
		$post->filename = null;
		$post->path 	= null;
		$post->size 	= null;
		$post->dt 		= null;
		
		if( !$folderId ) {
			$post->error = JText::_('COM_JEXTRANET_NO_SELECTED_FOLDER');
			return $post;
		} 
		
		if ( empty( $_FILES['mf_file_upload']) ) {
			$post->error = JText::_('COM_JEXTRANET_NO_SELECTED_FILE');
			return $post;
		}
		
		$folder 			= JExtranetHelper::getFolder( $folderId );
		$uploadDirectory 	= JExtranetHelper::getSystem()->getUploadFolder() . $folder->ftp;
		
		if (!is_writable($uploadDirectory)){
			$post->error = JText::_('COM_JEXTRANET_MULTI_UPLOAD_ERROR_UNWRITABLE_FOLDER');
			return $post;
		}
		
		if (!isset($_FILES['mf_file_upload'])) {
			$post->error = JText::_('COM_JEXTRANET_ERROR_UNABLE_TO_UPLOAD_FILE');
			return $post;
		}

		$file 				= array();
		$file['name'] 		= $_FILES['mf_file_upload']['name'];
		$file['tmp_name'] 	= $_FILES['mf_file_upload']['tmp_name'];
		$file['f_size']		= $_FILES['mf_file_upload']['size'];
	
		// check for files that starts with a dot like .htaccess
		$fileinfo 	= $this->getFileInfo( $file['name'] );
		
		$filename 	= mb_convert_encoding( $fileinfo->basename, 'UTF-8');
		$filename	= str_replace( array("/","\\"), '' , $filename );
		
		while (file_exists($uploadDirectory . DIRECTORY_SEPARATOR . $filename . '.' . $fileinfo->extension )) {
			$filename .= rand(10, 99);
		}
		
		$file['name'] 	= $fileinfo->extension ? $filename . '.' . $fileinfo->extension : $filename;
		$path 			= $uploadDirectory . DIRECTORY_SEPARATOR . $file['name'];
		
		if (version_compare(JVERSION, '3.4.0', 'ge'))
		{
			$hasUpload = JFile::upload($file['tmp_name'], $path, false, true);
		}
		else
		{
			$hasUpload = JFile::upload($file['tmp_name'], $path);
		}

		if ($hasUpload) 
		{ 
			// creating the file 
			$file['f_title'] 		= $fileinfo->basename;
			$file['f_type']			= $fileinfo->extension;
			$file['f_alias']		= $folder->ftp . DIRECTORY_SEPARATOR . $file['name'];
			$file['f_parent']		= $folder->id;
			$file['f_created']		= JExtranetHelper::getSystem()->getDateToMySQL();
			$file['f_created_by']	= JFactory::getUser()->get('id');
					
			usleep( $fileOrder*25000 );
			$file['ordering']	= $this->getOrderIndex( $folder ) + 1;
		
			$this->addToSql( $file );
			$post->filename = $file['name'];
			$post->path 	= $path;
			$post->size 	= JExtranetHelper::getSystem()->parseSize($file['f_size']);
			$post->dt 		= $file['f_created'];
			$post->Folders	= $this->getFolders( $folderId );
			$post->Files	= $this->getFiles( $folderId );
		}
		return $post;
	}
	
	
	protected function getFileInfo( $filename ) {

		$file = new stdClass;
		$file->basename 	= null;
		$file->extension 	= null;
		$file->length 		= strlen( $filename );

		if ( substr_count( $filename, '.') == 0 ) {
			$file->basename 	= $filename;
		}

		if ( substr_count( $filename, '.') == 1 ) {
			
			$regex 		= '/^\.[^.]*$/';

			// if file starts with a dot
			if ( preg_match( $regex, $filename ) ) {
				$file->basename = str_replace('.', '', $filename);
			} else {
				$parts = explode('.', $filename);
				$file->basename = $parts[0];
				$file->extension = $parts[1];
			}
		}


		if ( substr_count( $filename, '.') >= 2 ) {
			$pattern = '/.[^.]*$/';

			if ( preg_match( $pattern, $filename, $found ) ) {
				$file->extension = str_replace('.', '', $found[0]);
				$file->basename = substr( $filename, 0, ($file->length - strlen( $file->extension) - 1 ) );
			}

		} 

		return $file;
	}


	protected function getFolders( $folder ) {

		$query = $this->_db->getQuery(true);
		$query->select(  $this->_db->quoteName('fd_name') .' AS Title,'.  $this->_db->quoteName('fd_id') .' AS Id,'. $this->_db->quoteName('fd_type')  . ',' . $this->_db->quoteName('ordering') );
		$query->from( $this->_db->quoteName('#__extranet_folders') );
		$query->where(  $this->_db->quoteName('fd_parent') . ' = ' . (int) $folder );
		$query->order('fd_id ASC');
		
		$this->_db->setQuery( $query );
		$this->_db->query();
		
		$folders = $this->_db->loadAssocList();
		return $folders;
	}
	
	
	protected function getFiles( $folder ) {
		
		$query = $this->_db->getQuery(true);
		$query->select( $this->_db->quoteName('f_id') . ' AS Id,' . $this->_db->quoteName('f_title') . ' AS Title,' .  $this->_db->quoteName('f_type')  . ',' . $this->_db->quoteName('ordering') );
		$query->from( $this->_db->quoteName('#__extranet_files') );
		$query->where(  $this->_db->quoteName('f_parent') . ' = ' . (int) $folder );
		$query->order('f_id ASC');

		$this->_db->setQuery( $query );
		$this->_db->query();
		
		$files = $this->_db->loadObjectList();
		return $files;
	}	
	
	
	protected function addToSql( $file ) {
		
		$null 		= '';
				
		$query	= $this->_db->getQuery( true );
		$columns = array(
				$this->_db->quoteName('f_id'),
				$this->_db->quoteName('f_title'),
				$this->_db->quoteName('f_alias'),
				$this->_db->quoteName('f_type'),
				$this->_db->quoteName('f_parent'),
				$this->_db->quoteName('f_preview'),
				$this->_db->quoteName('f_size'),
				$this->_db->quoteName('f_created'),
				$this->_db->quoteName('f_created_by'),
				$this->_db->quoteName('f_hits'),
				$this->_db->quoteName('f_description'),
				$this->_db->quoteName('f_publish_up'),
				$this->_db->quoteName('f_publish_down'),
				$this->_db->quoteName('published'),
				$this->_db->quoteName('ordering'),
				$this->_db->quoteName('checked_out')
				);

		$values = 'NULL,' . $this->_db->Quote( $file['f_title'] ) . ',' . $this->_db->Quote( $file['f_alias'] ) . ',' . $this->_db->Quote( $file['f_type'] ) . ',' 
							. $file['f_parent'] . ',' . $this->_db->Quote($null) . ',' . $file['f_size'] . ',' . $this->_db->Quote( $file['f_created'] ) . ',' . $file['f_created_by']  . ',' 
							. $this->_db->Quote($null) . ',' . $this->_db->Quote($null) . ',' . $this->_db->Quote('0000-00-00 00:00:00') . ',' . $this->_db->Quote( '0000-00-00 00:00:00' ) . 
							',1,' . $file['ordering'] .',0'; 
		$query->insert('#__extranet_files');
		$query->columns( $columns );
		$query->values( $values );
		$this->_db->setQuery($query);
		$this->_db->query();
	}
	

	protected function getOrderIndex( $folder ) {
		        	
		$query = $this->_db->getQuery(true);
		
		$query->select('MAX(a.ordering)');
		$query->from('#__extranet_files AS a');
		$query->where('a.f_parent  = ' .  $folder->get('id') );
		$this->_db->setQuery($query);
		$ordering = $this->_db->loadResult();
	
		return $ordering;
	}
	
}
?>
