<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetHelperFolder extends JObject
{
	
	protected $_db;
	public $id;
	public $name;
	public $parent;
	public $type;
	public $ftp;
	public $alert;
	public $uploadBase;
	public $uploadPath;
	public $description;
	protected static $instance;
	
	
	public static function getInstance( $id ) {
		if( !isset(self::$instance[$id] ) ) {
			self::$instance[$id] = new self( $id );
		}
		return self::$instance[$id];
	}
	

	public function __construct( $id ) {
		
		$this->_db 			= JFactory::getDbo();
		$this->id			= $id;
		$this->name 		= null;
		$this->parent 		= 0;
		$this->type			= 'pv';
		$this->ftp			= null;
		$this->alert 		= 0;
		$this->published	= 0;
		$this->uploadBase 	= JExtranetHelper::getSystem()->getUploadFolder();
		$this->description 	= '';
		$this->getFolder();
	}
	

	/*
	 * Method to get folder details
	 */
	protected function getFolder() {
		
		$query = $this->_db->getQuery( true );
		$query->select('a.fd_id AS id, a.fd_parent AS parent, a.fd_name AS name,a.fd_type AS type, a.fd_name_ftp AS ftp, a.fd_alerts AS alert, a.published, a.fd_description as description');
		$query->from('#__extranet_folders AS a');
		$query->where('a.fd_id = ' . $this->id);

		$this->_db->setQuery($query);
		$this->setProperties( $this->_db->loadAssoc() );	
		$this->uploadPath	= $this->uploadBase . $this->ftp;	
	}


	/*
	 * Method to get all subfolders contained by a main folder
	 * @return		array 		an array of subfolders
	 */
	public function getSubfolders ( $recursive = false ) {

		$query	= $this->_db->getQuery(true);
	
		// get all folders
		$query->select('a.fd_id AS id, a.fd_name AS name, a.fd_parent AS parent, a.fd_name_ftp as ftp');
		$query->from('#__extranet_folders AS a ');
		$query->order('a.fd_parent ASC');
		$this->_db->setQuery( $query );
		$rez = $this->_db->loadObjectList ();
			
		$sub		= array();
		$subfolders = $this->get_child_nodes( $this->id, 0, $rez, $sub, $recursive );
		
		return $subfolders;
	}
	
	
	protected function get_child_nodes($parent, $level, $folders, &$subfolders, $recursive ) {
		
		foreach ( $folders as $folder ) {
			if ( $folder->parent == $parent ) {
				$folder->deep = $level;
				$subfolders[] = $folder;
				if ( $recursive) {
					$this->get_child_nodes( $folder->id, $level+1, $folders, $subfolders, $recursive );
				}
			}
		}
		
		return $subfolders;
	}
	

	// method to see if this folder is parent of needle
	public function isParentFor( $needle ){
			
		$childrens = $this->getSubfolders( true );
		foreach ( $childrens as $children ) {
			if ( $children->id == $needle ) {
				return true;
			}
		}
		return false;
	}
	
	
	/*
	 * Method to return parent ID, if recursive will return all parents until the root folder
	 * @return		array 		array containing id of all the parents of this folder
	 */
	public function getParent ( $recursive = false ) {
		
		// Initialize variables.
	 	$parents = array();
				
		$parents[] 	= $this->parent;
	
		if ( $recursive ) {
			$recursive = $this->parent;
			while ( $recursive ) { 
				$folder 	= JExtranetHelper::getFolder( $recursive );
				$parents[] 	= $folder->parent;
				$recursive 	= $folder->parent;
			}
		}
		return $parents;
	}


	public function getMainParent()
	{
		if ($this->parent == 0)
		{
			return $this->id;
		}
		else 
		{
			return self::getInstance($this->parent)->getMainParent();
		}
	}


	public function files_count($recursive = false)
	{
		$query	= $this->_db->getQuery(true);
		$query	-> select('COUNT(f_id)')
				-> from('#__extranet_files')
				-> where('f_parent = ' . $this->id);
		$this->_db->setQuery( $query );
		$count = $this->_db->loadResult();
        
        if ($recursive) {
            $subfolders = $this->getSubfolders();
            if (!empty($subfolders)) {
                foreach ($subfolders as $subfolder) {
                    $count += JExtranetHelper::getFolder($subfolder->id)
                                ->files_count($recursive);
                }  
            }
        }

        return $count;
	}
	
}
