<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');


jimport('joomla.client.helper');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

class JExtranetModelFile extends JModelList
{
	public		$file;
	protected  	$current;
	protected 	$params;
	protected	$user;
	protected	$base;
	protected 	$folder;
	protected  	$direct;
	protected 	$preview;

	public function __construct($config = array()) {
		
		$this->current 	= JExtranetHelper::getSystem()->getDateToMySQL();
		$this->base 	= JExtranetHelper::getSystem()->getUploadFolder();
		$this->params 	= JComponentHelper::getParams('com_jextranet');
		$this->direct	= JFactory::getApplication()->getUserState('direct.download.id', 0);
		$this->preview 	= JFactory::getApplication()->input->get('preview',0,'int');
		
		$this->file 			= new stdClass();
		$this->file->f_id		= JRequest::getVar('id','0','','int');
		$this->file->location 	= null;
		$this->file->name 		= null;
		
		$this->folder			= JRequest::getVar('folder', 0, '', 'int');
		$this->user				= JExtranetHelperUser::getInstance();


		JClientHelper::setCredentialsFromRequest('ftp');
		parent::__construct($config);
	}
	

	public function getFile () {
		
		clearstatcache();

		if ( $this->file->f_id ) {

			$query = $this->_db->getQuery( true );
			$query->select('a.f_id, a.f_title, a.f_alias, a.f_size, a.f_type,a.f_parent');
			$query->from('#__extranet_files AS a');
			$query->where('a.f_id = ' . $this->file->f_id);
			$query->where('a.published = 1');
			$query->where('((a.f_publish_up = "0000-00-00 00:00:00") OR (a.f_publish_up <= ' . $this->_db->Quote($this->current) .'))' );
			$query->where('((a.f_publish_down = "0000-00-00 00:00:00") OR (a.f_publish_down > '. $this->_db->Quote($this->current) .'))');
			
			$this->_db->setQuery($query);
			$file = $this->_db->loadObject();

			if (!empty($file)) {
				
				// if the user doesn't have the proper permissions exit
				// 3 - download, 5 - delete
				$levels = $this->user->getAuthorisationLevel( $file->f_parent );
				if ( !in_array(3,$levels) && !in_array(5,$levels) && !($this->direct == $this->file->f_id) ) {
					exit();
				}
				
				$this->file = $file;
				$this->file->filename = $this->file->f_title . '.' . $this->file->f_type;

				//if it's not an external file, get MIME
				if ( !$this->isExternal() ) {
					$this->file->external = false;
					$this->file->location = $this->base . DIRECTORY_SEPARATOR . $this->file->f_alias;

					if (JFile::exists( $this->file->location )) { 
						$this->getMime();
					}
				} else {
					$this->file->location = $this->file->f_alias;
					$this->file->external = true;
					$this->file->mime = 'application/octet-stream';
				}
			} 
		} 

		return $this->file; 
	}
	
	
	public function getMime() {
		
		if (function_exists('mime_content_type')) {
			$this->file->mime = mime_content_type($this->file->location);
		} elseif (function_exists('finfo_open')) {
			$finfo 		= finfo_open(FILEINFO_MIME);
			$mimeType 	= @finfo_file($finfo, $this->file->location);
			finfo_close($finfo);
			if (strpos($mimeType, ';')) { 
				list($mimeType,) = explode(';', $mimeType);
				$this->file->mime = $mimeType;
			} 
		} elseif (!empty($mime_type[strtolower($this->file->f_type)])) {
			$this->file->mime = $mime_type[strtolower($this->file->f_type)];
		} else {
			$this->file->mime = 'application/octet-stream';
		}			
	}
	
	
	// check if the location of the file contains the "://" string ( http://, https://, ftp://, sftp://, .... ) supported by fopen
	protected function isExternal() {
		
		$regex = '/^[A-Za-z]*:\/\/[^\/](.)*/';
		if ( preg_match( $regex, $this->file->f_alias ) ) {
			return true;
		}
		return false;
	}
	

	public function getHeaders() {

		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-Type: " . $this->file->mime);
		if ($this->file->f_size) {
			header("Content-Length: " .(string)($this->file->f_size) );
		}

		if(!$this->preview) 
		{
			header('Content-Disposition: attachment; filename="' . $this->file->filename . '"');
			header("Content-Transfer-Encoding: binary\n");
		}
	}


	// method to get a file content
	public function getContent () {

		if ( $memory = (int) $this->params->get( 'setMemoryLimit', 0 ) ) {
			$memory = $memory . 'M';
			@ini_set('memory_limit', $memory);
		}

		if ( !$this->file->external ) {
			// if file size is larger than 10MB
			if ( $this->file->f_size >= 10000000 ) {
				$this->getContentExternal();
			} else {
				$this->getContentInternal();
			}
		} else {
			$this->getContentExternal();
		}
	}


	// used for files stored on own server
	protected function getContentInternal() {
		ob_clean();
		flush();
		$data = JFile::read( $this->file->location );
		echo $data;
		ob_flush();
		flush(); 
		exit(0); 
	}


	// used for external links
	protected function getContentExternal() {

		$fh = fopen( $this->file->location, 'rb' );
		while ( !feof($fh) ) {
			ob_clean();
			flush();
			$data = fread($fh, '8192');
			echo $data;
			ob_flush();
			flush();
		}
		fclose($fh);
		exit(0);
	}
	

	public function setHits () {
		
		$query = $this->_db->getQuery ( true );
		$query->update('#__extranet_files');
		$query->set('f_hits = (f_hits + 1)');
		$query->where('f_id = ' . $this->file->f_id);
		$this->_db->setQuery($query);
		$this->_db->query();
	
	}


	public function setFileLogs () {

		$use	= $this->params->get('useFileLogs', 0);
		
		if ( $use ) {
			$logfile 		= JPATH_COMPONENT . DIRECTORY_SEPARATOR . 'logs' . DIRECTORY_SEPARATOR . 'files' . DIRECTORY_SEPARATOR .  $this->file->f_id . '.ini';
			$registry 	= new JRegistry();

			if (JFile::exists( $logfile )) {
				$registry->loadFile( $logfile );
				$registry->set( $this->current, $this->user->get('id') );
				JFile::write( $logfile, $registry );
			} else {
				$registry->set( $this->current, $this->user->get('id') );
				JFile::write( $logfile, $registry );
			}
		} 
	}
	
	
	public function setUserLogs () {
		
		$use	= $this->params->get('userLogs', 0);
		
		if ( $use ) {
			$logfile 		= JPATH_COMPONENT . DIRECTORY_SEPARATOR . 'logs' . DIRECTORY_SEPARATOR . 'users' . DIRECTORY_SEPARATOR . $this->user->get('id') . '.ini';
			$registry 	= new JRegistry();
			if (JFile::exists($logfile)) {
				$registry->loadFile( $logfile );
				$registry->set( $this->current, $this->file->f_id );
				JFile::write($logfile,$registry);
			} else {
				$registry->set( $this->current, $this->file->f_id );
				JFile::write($logfile,$registry);
			}
		} 
	}
	
	
	
	/*
	 * Method to bookmark file by user
	 * 
	 * @param			$file		ID of the file to be bookmarked	
	 */
	public function bookmark ( $file ) {
		
		$canDo	 	= $this->params->get( 'useBookmarks', 1 );
		$authLevel 	= $this->user->getAuthorisationLevel ( $this->folder );
		
		if ( ! $canDo || !in_array(2,$authLevel) ) { 
			return false;
		}
		
		$uparams = $this->user->getParams();
		if(empty($uparams)) {
			$uparams = new JObject;
		}

		if(empty($uparams->bookmarks)) {
			$uparams->bookmarks = array( $file );
		} else {
			if (!in_array( $file, $uparams->bookmarks) ) {
				$uparams->bookmarks[] = $file;
			}
		}
		
		$params = json_encode( $uparams );
		
		$query = $this->_db->getQuery(true);
		$query->update('#__extranet_users');
		$query->set( $this->_db->quoteName('u_params') . ' = ' . $this->_db->Quote($params) );
		$query->where( $this->_db->quoteName('u_id') . ' = ' . $this->user->get('id') );
		$this->_db->setQuery($query);
		$this->_db->query();

		return true;
	}
	
	
	
	
	
	/*
	 * Method to upload file into folders (folder=0 - user folder, folder!=0 extranet folders)
	 * 
	 * name: 		upload
	 * 
	 * @param
	 * @return		boolean		true if upload successfully
	 */
	public function upload($file, $id) 
	{
		
		$post			= new stdClass();
				
		$post->error 	= null;
		$post->filename = null;
		$post->path 	= null;
		$post->size 	= null;
		$post->dt 		= null;
		
		// check authorizations
		if (!empty($this->folder)) { 
			$level = $this->user->getAuthorisationLevel( $this->folder );

			//  if he doesn't have right to allow into the folder
			if ( !in_array(4,$level)) {
				$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_FOLDER_PROHIBITED');
				return $post;					
			} 			
		} else {
			
			$uparams = $this->user->getParams();
			
			// Check if upload into his own folder is enabled
			if ( !$this->params->get( 'allowUserUploadOwnFolder', 0 ) == 1 && empty($uparams->allowUserUploadOwnFolder) ) {
				$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_FOLDER_PROHIBITED');
				return $post;				
			}
		}
		
		$file['name'] 		= JFile::makeSafe($file['name']);
		$pathinfo 			= pathinfo($file['name']);
		$original 			= $pathinfo['filename']; 
		$file['f_type']		= @$pathinfo['extension'];

		if ( empty( $file['name'] ) ) {
			$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_INVALID_REQUEST');
			return $post;		
		}

			
		// get the $filepath
		if ($this->folder == 0) {
			
			$userbase = $this->base . DIRECTORY_SEPARATOR .'extranet_ufiles'  . DIRECTORY_SEPARATOR . $this->user->get('id');

			if (!JFolder::exists( $userbase )) {
				JFolder::create( $userbase, 0750 );
			}
			
			$file['f_alias']	= $file['name']; 
			$file['path'] 		= JPath::clean( $userbase . DIRECTORY_SEPARATOR . $file['name'] );
			
		} else {

			$fd_name = JExtranetHelper::getFolder( $this->folder )->ftp;
			
			if (empty($fd_name)) {
				$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_FOLDER_PROHIBITED');
				return $post;	
			}
			
			$file['f_alias']	= $fd_name . DIRECTORY_SEPARATOR . $file['name'];
			$file['path'] 		= JPath::clean($this->base . DIRECTORY_SEPARATOR . $fd_name . DIRECTORY_SEPARATOR . $file['name']);
		}
			
			
		// Check if extension is valid
		$allowed 		= explode(',', $this->params->get( 'allowedExtensions', 'doc,docx,xls,xlsx,ppt,pps,pptx,ppsx,odt,odf,odp,jpg,jpeg,pdf,txt,png,zip' ) );
		$nonallwed 		= explode(',', $this->params->get( 'restrictedExtensions', 'php,html' ) );
		$allowedmime	= explode(',', $this->params->get( 'allowedMimeType', 'application/vnd.ms-office,application/msword,application/vnd.openxmlformats-officedocument.wordprocessingml.document,application/vnd.ms-excel,application/vnd.openxmlformats-officedocument.spreadsheetml.sheet,application/vnd.ms-powerpoint,application/vnd.openxmlformats-officedocument.presentationml.presentation,application/vnd.openxmlformats-officedocument.presentationml.slideshow,application/vnd.oasis.opendocument.text,application/vnd.oasis.opendocument.formula,application/vnd.oasis.opendocument.presentation,image/jpeg,application/pdf,text/plain,image/png,application/x-zip,application/zip' ) );
					
				
		// check if file extension/MIME is allowed
		if ($this->params->get('disableMimeChecking') != 1)
		{		
			if (!in_array($file['f_type'],$allowed) || in_array($file['f_type'],$nonallwed)) 
			{
				$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_EXTENSION_PROHIBITED');
				return $post;		
			}

			if ( class_exists('finfo') ) {
				$finfo = finfo_open(FILEINFO_MIME_TYPE);
				$mime = finfo_file($finfo, $file['tmp_name']);
			} elseif ( function_exists('mime_content_type') ) {
				$mime = mime_content_type ( $file['tmp_name'] );
			} else {
				$mime = null;
			}

			if ( $mime && !in_array( $mime, $allowedmime )) {
				$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_EXTENSION_PROHIBITED');
				return $post;	
			}
		}

		if (JFile::exists($file['path'])) 
		{
			$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_FILE_EXISTS');
			return $post;
		} 

		if (version_compare(JVERSION, '3.4.0', 'ge'))
		{
			$hasUpload = JFile::upload($file['tmp_name'], $file['path'], false, true);
		}
		else
		{
			$hasUpload = JFile::upload($file['tmp_name'], $file['path']);
		}

		if (!$hasUpload) 
		{
			$post->error = JText::_('COM_JEXTRANET_FILE_UPLOAD_ERROR_UNABLE_TO_UPLOAD_FILE');
			return $post;
		} 
		else 
		{
			// remove the extension from name
			$file['name'] 	= $original;
			// create the response
			$post->filename = $file['name'];
			$post->path 	= $file['path'];
			$post->size 	= JExtranetHelper::getSystem()->parseSize($file['size']);
			$post->dt 		= $this->current;	

			// add the file to db		
			$this->folder ? $this->insertFoldersRecord( $file ) : $this->insertVaultRecord( $file );
			
			// send possible alerts
			$this->sendAlert ( $file['name'] );
		}
		
		return $post;
	}
	
	
	protected function insertFoldersRecord( $file ) {

		$query	= $this->_db->getQuery( true );
		
		$query->select('MAX(a.ordering)');
		$query->from('#__extranet_files AS a');
		$query->where('a.f_parent = ' . $this->folder);
		$this->_db->setQuery( $query );
		$max = $this->_db->loadResult();
		$max = $max ? ( $max + 1) : 1;
		
		$columns = array( 
				$this->_db->quoteName('f_id'),
				$this->_db->quoteName('f_title'),
				$this->_db->quoteName('f_alias'),
				$this->_db->quoteName('f_type'),
				$this->_db->quoteName('f_parent'),
				$this->_db->quoteName('f_preview'),
				$this->_db->quoteName('f_size'),
				$this->_db->quoteName('f_created'),
				$this->_db->quoteName('f_created_by'),
				$this->_db->quoteName('f_hits'),
				$this->_db->quoteName('f_description'),
				$this->_db->quoteName('f_publish_up'),
				$this->_db->quoteName('f_publish_down'),
				$this->_db->quoteName('published'),
				$this->_db->quoteName('ordering')
				);
		$values = 
				'NULL, ' . 
				$this->_db->Quote( $file['name'] ) . ',' . 
				$this->_db->Quote( $file['f_alias'] ) .',' . 
				$this->_db->Quote( $file['f_type'] ) . ',' . 
				$this->folder . ',' .
				$this->_db->Quote('') . ','  .
				$file['size'] . ',' . 
				$this->_db->Quote($this->current) .',' . 
				$this->user->get('id') .',' . 
				'0,' . 
				$this->_db->Quote('') . ','  .
				$this->_db->Quote('') . ','  .
				$this->_db->Quote('') . ','  .
				'1,' . 
				$max;

		$query->clear();
		$query->insert('#__extranet_files');
		$query->columns( $columns );
		$query->values( $values );
		$this->_db->setQuery($query);
		$this->_db->query();
					
	}
	
	
	protected function insertVaultRecord( $file ) {
		
		$query	= $this->_db->getQuery( true );
		
		$columns = array( 
				$this->_db->quoteName('f_id'),
				$this->_db->quoteName('f_title'),
				$this->_db->quoteName('f_alias'),
				$this->_db->quoteName('f_type'),
				$this->_db->quoteName('f_size'),
				$this->_db->quoteName('f_created'),
				$this->_db->quoteName('f_created_by'),
				$this->_db->quoteName('f_hits'),
				$this->_db->quoteName('f_description')
				);
		$values = 
				'NULL, ' . 
				$this->_db->Quote($file['name']) . ',' . 
				$this->_db->Quote($file['f_alias']). ',' . 
				$this->_db->Quote( $file['f_type'] ). ',' . 
				 $file['size'] . ',' . 
				$this->_db->Quote($this->current) . ',' . 
				$this->user->get('id') . 
				',0,' . 
				$this->_db->Quote('');

		$query->insert('#__extranet_users_files');
		$query->columns($columns);
		$query->values($values);
		$this->_db->setQuery($query);
		$this->_db->query();
	}
	
	
	protected function sendAlert ( $filename ) {
		
		if ( $this->folder ) {
			$folder = JExtranetHelper::getFolder( $this->folder )->name;
		} else {
			$folder = JText::_('COM_JEXTRANET_USER_FOLDER');
		}
		
		
		$send 		= $this->params->get( 'notifyAdmin', 0 );
		$method 	= $this->params->get( 'notifyAdminMethod', 2 );
		$subject 	= JText::sprintf( JText::_('COM_JEXTRANET_NEW_FILE'), $this->user->get('name'), $filename );
		$body		= $this->params->get( 'notifyAdminBody' );
		$body		= $this->prepareMessage($body,$filename, $folder);
		
		if ( $send ) {
			
			switch ($method) {
			
				case ('0'):
					$this->sendMail($subject,$body);
				break;
				
				case ('1'):
					$this->sendMessage($subject, $body);
				break;
			
				case ('2'):
					$this->sendMessage($subject, $body);
					$this->sendMail($subject,$body);
				break;
				
			}
		}
	}
	
	
	
	
	/*
	 * Method to prepare the message to be sent ( replace possible tags )
	 * Used by sendAlerts function
	 * 
	 * name: 		prepareMessage
	 * 
	 * @param		string $message		the message to be sent
	 * 				string $filename	the name of the file to add
	 * 				int $folder			the name of the folder where the file is added
	 * 
	 * @return		string				the message					
	 */
	protected function prepareMessage ( $message, $filename, $folder ) {

		$website 	= JFactory::getApplication('administrator')->getCfg('sitename');
		$filename 	= $filename;
		$folder 	= $folder;
		
		$message = str_replace ('__WEBSITE__',$website,$message);
		$message = str_replace ('__FILENAME__',$filename,$message);
		$message = str_replace ('__FOLDER__',$folder,$message);
		$message = str_replace ('__DATE__',$this->current,$message);
		
		return $message;
		
	}
	
	
	/*
	 * Method to send the alert by mail
	 * Used by sendAlerts function
	 * 
	 * name: 		sendMail
	 * @param		string $subject		the subject for the mail/alert
	 * 				string $body		the body for the mail/alert
	 * @return				
	 */
	protected function sendMail ( $subject, $body ) {
		
		$recipient 	= $this->params->get('notifyAdminEmail', JFactory::getApplication('administrator')->getCfg('mailfrom'));
		$from 		= $this->user->get('email');
		$fromname	= $this->user->get('name');
		$cc 		= $this->params->get( 'notifyCCEmail', null );
        $method = $this->params->get('multipleEmailsMethod', 0);
        $bcc = null;

        if ($method == 0) {
            $bcc = $cc;
            $cc = null;
        }

        JFactory::getMailer()->sendMail(
            $from, $fromname, $recipient, $subject, $body, true, $cc, $bcc);
	}
	
	
	
	/*
	 * Method to store the message
	 * Used by sendAlerts function
	 * 
	 * name: 		sendMessage
	 * @param		string $subject		the subject for the alert
	 * 				string $body		the body for the alert
	 * @return				
	 */
	protected function sendMessage ( $subject, $body ) {
		
		$from = $this->user->get('id');

		$columns = array( 	$this->_db->quoteName('m_id'),  
							$this->_db->quoteName('m_user_id_from'),
							$this->_db->quoteName('m_user_id_to'), 
							$this->_db->quoteName('m_date_time'), 
							$this->_db->quoteName('published'), 
							$this->_db->quoteName('m_subject'),
							$this->_db->quoteName('m_message')
						);

		$query = $this->_db->getQuery ( true );
		$query->insert( $this->_db->quoteName('#__extranet_messages') );
		$query->columns( $columns );
		$query->values('NULL,' . $from . ', 1,' . $this->_db->Quote($this->current) . ',0, '. $this->_db->Quote($subject) . ', ' . $this->_db->Quote($body) );
		$this->_db->setQuery($query);
		$this->_db->query();
	}
	
	
	public function fdelete () {
		$file = $this->getFile();
		
		if (! JFile::delete($file->location ) ) {
			$this->setError( JText::_('COM_JEXTRANET_FOLDERS_DELETE_FILE_ERROR') );
			return false;
		}

		$query = $this->_db->getQuery(true);
		$query->delete('#__extranet_files');
		$query->where( 'f_id = ' . $file->f_id );

		$this->_db->setQuery ( $query );
		$this->_db->query();
		
		return true;
	}
	
	
}
?>
