<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');

class JExtranetModelFolders extends JModelLegacy
{
	protected	$folder;
	protected 	$user;	
	protected	$params;
	protected static $items;
	protected $start;
	protected $perpage = 50;

	public function __construct($config = array()) {
		$id 			= JRequest::getVar('fid','0','','int');
		$this->folder 	= JExtranetHelper::getFolder( $id );
		$this->user 	= JExtranetHelperUser::getInstance();
		$this->params	= JComponentHelper::getParams('com_jextranet');
        $this->start = JFactory::getApplication()->input->get('start',0,'int');

		parent::__construct($config);
	}

    protected function query() {
        $query =
            'SELECT title, description, id, type, size, ordering, creation '.
            'FROM ('.
                'SELECT a.f_title AS title, a.f_description AS description,'.
                    'a.f_id AS id, a.f_type AS type, a.f_size AS size,'.
                    'a.ordering as ordering, a.f_created as creation '.
                'FROM #__extranet_files AS a '.
                'WHERE a.f_parent = '.$this->folder->get('id'). ' '.
                'UNION ALL '.
                'SELECT b.fd_name AS title, b.fd_description AS description,'.
                    'b.fd_id AS id, "jextranetfolder" AS type, 0 AS size,'.
                    'b.ordering as ordering, b.fd_created as creation '.
                'FROM #__extranet_folders AS b '.
                'WHERE b.fd_parent = '.$this->folder->get('id').
            ') t';
            return $query;
    }

	protected function items(){
        if(!isset(self::$items)) {
            $this->_db->setQuery($this->query());
            $items = $this->_db->loadObjectList();
            $splited = array('folders'=>array(), 'files'=>array());
            foreach ($items as $key => $item) {
                $permissions = $this->user->getAuthorisationLevel($item->id);
                if ($item->type == 'jextranetfolder' &&
                    in_array(2, $permissions)) {
                    $splited['folders'][$key] = $item;
                }
                if ($item->type !== 'jextranetfolder') {
                    $splited['files'][$key] = $item;
                }
            }
            self::$items = array_merge($splited['folders'], $splited['files']); 
        }
        $total = count(self::$items);
        $this->pagination = new JPagination($total, $this->start, $this->perpage);
        $items = array_slice(self::$items, $this->start, $this->perpage);
        return $items;
	}

    public function getPagination()
    {
        return $this->pagination;
    }

	public function getFiles() {
		$files 	= array();
		if (!$this->folder->id || ($this->folder->id && $this->folder->published))
		{
			$items 	= $this->items();
			if(!empty($items)) {
				foreach ( $items as $item ) {
					if ( $item->type !== 'jextranetfolder' ) {
						$files[] = $item;
					}
				}
			}
		}
	
		if (!empty($files)) 
		{
			usort($files, function($a, $b){
                $florder = array('order' => 'ordering', 'size' => 'size',
                                    'name' => 'title', 'date' => 'creation');
				$orderfile = $florder[$this->params->get('filesOrder', 'order')];
				$orderdir = $this->params->get('filesOrderDir', 'ASC');

				switch ($orderfile) 
				{
					case 'ordering':
					case 'size':
                        return ($orderdir == 'ASC') ?
                            $a->{$orderfile} > $b->{$orderfile}:
                            $a->{$orderfile} < $b->{$orderfile}; 
					break;
					
					case 'creation':
                        return ($orderdir == 'ASC') ?
                            strtotime($a->creation) > strtotime($b->creation):
                            strtotime($a->creation) < strtotime($b->creation);
					break;
					
					case 'title':
						if (function_exists('iconv'))
						{
                            $x = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE',
                                $a->title);
                            $y = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE',
                                $b->title);
                            $r = strcasecmp($x, $y);
						}
						else
						{
							$r = strcasecmp($a->title, $b->title);	
						}
						return ($orderdir == 'ASC') ? $r : (-1) * $r;
					break;
				}
			});
		}

		return $files;
	}

	public function getFolders() {
		$folders 	= array();
		if (!$this->folder->id || ($this->folder->id && $this->folder->published))
		{
			$items 		= $this->items();
			if(!empty($items)) {
				foreach ( $items as $item ) {
                    if ($item->type == 'jextranetfolder'){
						$folders[] = $item;
					}
				}
			}
		}
		if (!empty($folders)) 
		{
			usort($folders, function($a, $b){

				$fdorder 	= array('order' => 'ordering', 'name' => 'title', 'date' => 'creation');
				$orderfolder = $fdorder[$this->params->get('folderOrder', 'order')];
				$orderdir 	= $this->params->get('folderOrderDir', 'ASC');

				switch ($orderfolder) 
				{
					case 'ordering':
                        return ($orderdir == 'ASC') ?
                            $a->ordering > $b->ordering:
                            $a->ordering < $b->ordering; 
					break;
					
					case 'creation':
                        return ($orderdir == 'ASC') ?
                            strtotime($a->creation) > strtotime($b->creation):
                            strtotime($a->creation) < strtotime($b->creation);
					break;
					
					case 'title':
						if (function_exists('iconv'))
						{
                            $x = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE',
                                $a->title);
                            $y = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE',
                                $b->title);
                            $r = strcasecmp($x, $y);
						}
						else
						{
							$r = strcasecmp($a->title, $b->title);	
						}
						return ($orderdir == 'ASC') ? $r : (-1) * $r;
					break;
				}
			});
		}

		return $folders;
	}

	public function getRoute () {

		$route = array();

		if ( $this->folder->get('id') ) {
			$route[0] = new JObject();
			$route[0]->id = $this->folder->get('id');
			$route[0]->name = $this->folder->get('name');
		}

		$parent = $this->folder->get('parent');

		$i = 1;
		if (  $parent ) {
			while( $parent ) {
				$route[$i] = new stdclass;
				$route[$i]->id = $parent;
				$route[$i]->name = JExtranetHelper::getFolder($parent)->name;
				$parent = JExtranetHelper::getFolder($parent)->parent;
				$i++;
			}
		} 

		$home= new stdClass();
		$home->id = 0;
		$home->name = JText::_('COM_JEXTRANET_HOME');
		
		array_push( $route, $home );
		$route = array_reverse( $route );

		return $route;	
	}
	
	
	public function getParams() {
		return $this->params;
	}

	public function getFolder() {
		return $this->folder;
	}

	public function getUser() {
		return $this->user;
	}

	public function getTemplate () {
		
        $template = JFactory::getApplication()
                        ->getUserState('com_jextranet.template.folders');
		
		if ( !$template ) {
			$template = 'table';
		}

		return $template;
	}

	/*
	*	$form 	array 	$form['name'] - name for the new folder
	*					$form['folder'] - id for the parent folder
	*					$form['status'] - pb/pv - type for the new folder (public,private)
	*/
	public function create($form) {

		$base		= JExtranetHelper::getSystem()->getUploadFolder();
		$date		= JExtranetHelper::getSystem()->getDateToMySQL();	
		$parent		= JExtranetHelper::getFolder( $form['folder'] ); 

		$name 		= JFile::makeSafe($form['name']);
		$name 		= trim(str_replace (' ','_',$name));
		
		$relpath	= JPath::clean ( $parent->get('ftp') . DIRECTORY_SEPARATOR . $name ); 
		$path		= $base . $relpath;
	

		// create folders on server
		if (!JFolder::exists( $path )) {
			JFolder::create ( $path, 0750 );
		} else {
			$i = 1; 
			$path =  $base . $relpath . $i;
			while (JFolder::exists($path)){
				$i++;
				$path = $base . $relpath . $i; 
			}
			JFolder::create($path, 0750 );
		}

		// update relpath if more folder with same name
		 $relpath = !empty($i) ? $relpath . $i :  $relpath ;

		//get maximum for ordering
        $query 	= 'SELECT MAX(a.ordering) FROM #__extranet_folders AS a '.
                    'WHERE a.fd_parent = ' . $parent->get('id');
		$this->_db->setQuery ($query);
		$max = $this->_db->loadResult();


		// add new folder to database
		$query 		= $this->_db->getQuery(true);
		$columns 	= array ( 
						$this->_db->quoteName('fd_id'),
						$this->_db->quoteName('fd_parent'),
						$this->_db->quoteName('fd_name'),
						$this->_db->quoteName('fd_name_ftp'),
						$this->_db->quoteName('fd_description'),
						$this->_db->quoteName('fd_created'),
						$this->_db->quoteName('fd_type'),
						$this->_db->quoteName('fd_alerts'),
						$this->_db->quoteName('published'),
						$this->_db->quoteName('ordering'),
						$this->_db->quoteName('checked_out'),
						);
        $values	= 'NULL, '. $parent->get('id')   .', '.
            $this->_db->quote($this->_db->escape($form['name'])).', '.
            $this->_db->quote($this->_db->escape($relpath)) .',' .
			$this->_db->quote('') . ',' . $this->_db->quote($date) . ',' .
            $this->_db->quote($this->_db->escape( $form['status'] )) . ',' .
			'0, 1, '. ($max+1) .', 0';
		$query->insert('#__extranet_folders');
		$query->columns( $columns );
		$query->values( $values );
		$this->_db->setQuery( $query );
		$this->_db->query();
		return true;
	}	
}
?>
